import os
import time
import random
import keyboard  # pip instal keyboard
import config  # локальный


class Game:
    def __init__(self) -> None:
        self.player = Player(self)
        self.track = Track(config.ROWS, config.COLS, self.player)
        self.is_running = False
        self.last_update_time = None
        keyboard.on_press(self.get_key_pressed)
        self.start()

    def get_key_pressed(self, event):
        if event.name in config.KEYS:
            self.player.move(event)

    def start(self) -> None:
        self.is_running = True
        self.last_update_time = time.time()
        self.player.last_update_time = time.time()
        while self.is_running:
            if time.time() - self.last_update_time > 1 / config.FPS:
                self.make_game_frame()
                self.last_update_time = time.time()
        print('Врезался')

    def make_game_frame(self):
        os.system('cls')
        self.player.update()
        self.track.update()
        self.track.draw()

class Player:
    def __init__(self, game: Game) -> None:
        self.game = game
        self.x = config.COLS // 2
        self.y = config.ROWS - 1
        self.image = config.PLAYER
        self.last_update_time = None
        self.score = 0

    def __str__(self) -> str:
        return self.image

    def draw(self) -> None:
        pass

    def update(self):
        if self.x < 1 or self.x > self.game.track.cols - 2:
            self.game.is_running = False

        for obstacle in self.game.track.obstacles:
            if self.x == obstacle.x and self.y == obstacle.y:
                self.game.is_running = False

    def move(self, event) -> None:
        '''Двигает игрока'''
        if event.name == 'left':
            dx = -1
        elif event.name == 'right':
            dx = 1

        self.x += dx
        self.update()


class Track:
    def __init__(self, rows: int, cols: int, player: Player) -> None:
        self.lines = []
        self.rows = rows
        self.cols = cols
        self.player = player
        self.prev_player_x = None
        self.prev_player_y = None
        self.obstacle_id = 0
        self.obstacles = []
        for row in range(self.rows):
            line = [config.BORDER] + [config.EMPTY for cells in range(self.cols - 2)] + [config.BORDER]
            self.lines.append(line)

    def update(self) -> None:
        self.update_obstacles()
        self.spawn_obstacle()
        if self.prev_player_x is not None and self.prev_player_y is not None:
            self.lines[self.prev_player_y][self.prev_player_x] = config.EMPTY
        self.lines[self.player.y][self.player.x] = self.player
        self.prev_player_x = self.player.x
        self.prev_player_y = self.player.y

    def draw(self) -> None:
        for obstacle in self.obstacles:
            self.lines[obstacle.y][obstacle.x] = obstacle.image
        for line in self.lines:
            for cell in line:
                print(cell, end='')
            print()
        print(str(self.player.score).center(config.COLS, '-'))

    def spawn_obstacle(self):
        if self.obstacle_id % 6:
            obstacle = Obstacle(random.randint(1, self.cols - 2))
            self.obstacles.append(obstacle)
        self.obstacle_id += 1

    def update_obstacles(self):
        dy = 1
        obstacles_to_remove = []
        for obstacle in self.obstacles:
            if obstacle.y + dy > self.rows - 1:
                self.lines[obstacle.y][obstacle.x] = config.EMPTY
                obstacles_to_remove.append(obstacle)
                self.player.score += 1
                continue
            # Clear the obstacle from the previous line
            self.lines[obstacle.y][obstacle.x] = config.EMPTY
            obstacle.y += dy
            # Update the obstacle in the new position
            self.lines[obstacle.y][obstacle.x] = obstacle.image

        for obstacle in obstacles_to_remove:
            self.obstacles.remove(obstacle)

class Obstacle:
    def __init__(self, x):
        self.x = x
        self.y = 0
        self.image = config.OBSTACLE


if __name__ == '__main__':
    Game()
